
exports.up = function (knex, Promise) {
    return knex.schema.createTable('events', function (table) {
        table.inherits('messages');
        table.string('location').notNullable();
        table.dateTime('start_time').notNullable();
        table.dateTime('end_time').notNullable(); //TODO: add a CHECK (https://www.postgresql.org/docs/current/ddl-constraints.html) that start_time < end_time

        //TODO: for now, we support only 1 author (where as graphql schema indicates support for [Group] authors)
        table.string('author', 128).notNullable()
            .references('gid').inTable('groups')
            .onDelete('CASCADE'); //delete message if author is deleted
        table.integer('for_announcement')
            .references('mid').inTable('messages' /*'announcements'*/)
            .onDelete('SET NULL');
        /*
        ^ as stated in https://www.postgresql.org/docs/10/static/ddl-inherit.html#DDL-INHERIT-CAVEATS,
        "A serious limitation of the inheritance feature is that indexes (including unique constraints) and foreign key constraints only apply to single tables, not to their inheritance children."
        it is not possible (with PostgresQL v10) to use both inheritance and foreign keys at once.
        until it is, or a workaround is found, we simply don't maintain child-level referential integrity
        and we hope all the rest will work, e.g. simple_group_gids will indeed refer to simple groups, and idem for meta_group_gids.
        TODO: maybe, check if PostgresQL's latest version enables this feature?
        */
        
        //TODO: add support for participatingGroups and participatingUsers from the graphql schema

    }).then( () => {
        //update 'announcements' table by adding the 'for_event' column
        knex.schema.table('announcements', function (table) {
            table.integer('for_event')
                .references('mid').inTable('messages' /*'events'*/)
                .onDelete('SET NULL');
        });
        /*
        ^ as stated in https://www.postgresql.org/docs/10/static/ddl-inherit.html#DDL-INHERIT-CAVEATS,
        "A serious limitation of the inheritance feature is that indexes (including unique constraints) and foreign key constraints only apply to single tables, not to their inheritance children."
        it is not possible (with PostgresQL v10) to use both inheritance and foreign keys at once.
        until it is, or a workaround is found, we simply don't maintain child-level referential integrity
        and we hope all the rest will work, e.g. simple_group_gids will indeed refer to simple groups, and idem for meta_group_gids.
        TODO: maybe, check if PostgresQL's latest version enables this feature?
        */
    });
};

exports.down = function (knex, Promise) {
    return knex.schema.dropTable('events');
};
